//
//  View.swift
//  Pong
//
//  Created by Mark Meretzky on 10/7/14.
//  Copyright (c) 2014 Mark Meretzky. All rights reserved.
//

import UIKit

class View: UIView {
	let paddle: UIView;
	let ball: UIView;
	var dx, dy: CGFloat;	//how far the ball moves each time move is called.

	required init(coder aDecoder: NSCoder) {
		//Create the paddle.
		var frame: CGRect = CGRectMake(40, 100, 20, 100);
		paddle = UIView(frame: frame);
		paddle.backgroundColor = UIColor.whiteColor();

		//Create the ball in the upper left corner of this View.
		frame = CGRectMake(0, 0, 20, 20);
		ball = UIView(frame: frame);
		ball.backgroundColor = UIColor.whiteColor();

		//Ball initially moves to lower right.
		dx = 2;
		dy = 2;

		super.init(coder: aDecoder);
		backgroundColor = UIColor.greenColor();
		addSubview(paddle);
		addSubview(ball);
	}

	//Change the ball's direction of motion
	//if necessary to avoid an impending collision.
	
	func bounce() {
		//Where the ball would be if its horizontal motion were allowed
		//to continue for one more move.
		var horizontal: CGRect = ball.frame;
		horizontal.origin.x += dx;
	
		//Where the ball would be if its vertical motion were allowed
		//to continue for one more move.
		var vertical: CGRect = ball.frame;
		vertical.origin.y += dy;

		//Ball must remain inside bounds of big green View.
		if !CGRectEqualToRect(horizontal, CGRectIntersection(horizontal, bounds)) {
			//Ball will bounce off left or right edge of View.
			dx = -dx;
		}

		if !CGRectEqualToRect(vertical, CGRectIntersection(vertical, bounds)) {
			//Ball will bounce off top or bottom edge of View.
			dy = -dy;
		}

		//If the ball is not currently intersecting the paddle,
		if (!CGRectIntersectsRect(ball.frame, paddle.frame)) {

			//but will intersect the paddle on the next move,
			if CGRectIntersectsRect(horizontal, paddle.frame) {
				dx = -dx;
			}
	
			if (CGRectIntersectsRect(vertical, paddle.frame)) {
				dy = -dy;
			}
		}
	}

	override func touchesMoved(touches: NSSet, withEvent event: UIEvent) {
		let touch: UITouch = touches.anyObject() as UITouch;
		let point: CGPoint = touch.locationInView(self);
		var y: CGFloat = point.y;
		
		//Don't let the paddle move off the bottom or top of the View.
		let half: CGFloat = paddle.bounds.size.height / 2;
		y = min(y, bounds.size.height - half); //Don't let y get too big
		y = max(y, half);                      //or too small.
		
		paddle.center = CGPointMake(paddle.center.x, y);
		bounce();
	}

	func move(displayLink: CADisplayLink) -> Void {
		//println("\(displayLink.timestamp)");
	
		ball.center = CGPointMake(ball.center.x + dx, ball.center.y + dy);
		bounce();
	}

	/*
	// Only override drawRect: if you perform custom drawing.
	// An empty implementation adversely affects performance during animation.
	override func drawRect(rect: CGRect)
	{
		// Drawing code
	}
	*/

}
